"""
Intan driver module.
"""
from .base import BaseDriver

class IntanDriver(BaseDriver):
    """
    Driver for Intan hardware.
    """

    def execute_program(self, bioasm):
        """
        Executes a bio-assembly program using Intan.

        Args:
            bioasm: The bio-assembly program to execute.
        """
        pass
